﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.Drawing;

namespace howto_draw_equations
{
    // Draw one item over another.
    class FractionEquation : Equation
    {
        // True to draw a separator line.
        public bool DrawSeparator;

        // The space between the top and bottom items.
        private const float Gap = 0;

        // Extra width for the separator (on each side).
        private const float ExtraWidth = 6;

        // The items to draw.
        private Equation Numerator, Denominator;

        // Initialize a new object.
        public FractionEquation(Equation top_item, Equation bottom_item, bool draw_separator)
        {
            Numerator = top_item;
            Denominator = bottom_item;
            DrawSeparator = draw_separator;
        }

        // Initialize a new object.
        public FractionEquation(string top_string, string bottom_string, bool draw_separator)
            : this(new StringEquation(top_string), new StringEquation(bottom_string), draw_separator)
        {
        }

        // Set font sizes for sub-equations.
        public override void SetFontSizes(float font_size)
        {
            FontSize = font_size;
            Numerator.SetFontSizes(font_size * 0.75f);
            Denominator.SetFontSizes(font_size * 0.75f);
        }

        // Return the object's size.
        public override SizeF GetSize(Graphics gr, Font font)
        {
            // Get the sizes of the items.
            SizeF top_size, bottom_size;
            float width, height;
            GetSizes(gr, font, out top_size, out bottom_size, out width, out height);

            // Calculate our size.
            return new SizeF(width, height);
        }

        // Draw the equation.
        public override void Draw(Graphics gr, Font font, Pen pen, Brush brush, float x, float y)
        {
            // Get the sizes of the items.
            SizeF top_size, bottom_size;
            float width, height;
            GetSizes(gr, font, out top_size, out bottom_size, out width, out height);

            // Draw the separator.
            if (DrawSeparator)
            {
                float separator_y = y + top_size.Height + Gap / 2;
                gr.DrawLine(pen,
                    x, separator_y,
                    x + width, separator_y);
            }

            // Draw the top.
            float top_x = x + (width - top_size.Width) / 2;
            Numerator.Draw(gr, font, pen, brush, top_x, y);

            // Draw the bottom.
            float bottom_x = x + (width - bottom_size.Width) / 2;
            float bottom_y = y + top_size.Height + Gap;
            Denominator.Draw(gr, font, pen, brush, bottom_x, bottom_y);
        }

        // Return various sizes.
        private void GetSizes(Graphics gr, Font font, out SizeF top_size, out SizeF bottom_size, out float width, out float height)
        {
            top_size = Numerator.GetSize(gr, font);
            bottom_size = Denominator.GetSize(gr, font);
            width = Math.Max(top_size.Width, bottom_size.Width) + 2 * ExtraWidth;
            height = top_size.Height + bottom_size.Height + Gap;
        }
    }
}
