﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.Drawing;

namespace howto_draw_equations
{
    // Draw an item to a power.
    class PowerEquation : Equation
    {
        // The items to draw.
        private Equation Base, Power;

        // Initialize a new object.
        public PowerEquation(Equation base_item, Equation power_item)
        {
            Base = base_item;
            Power = power_item;
        }

        // Initialize a new object.
        public PowerEquation(string base_string, string power_string)
            : this(new StringEquation(base_string), new StringEquation(power_string))
        {
        }

        // Set font sizes for sub-equations.
        public override void SetFontSizes(float font_size)
        {
            FontSize = font_size;
            Base.SetFontSizes(font_size);
            Power.SetFontSizes(font_size * 0.75f);
        }

        // Return the object's size.
        public override SizeF GetSize(Graphics gr, Font font)
        {
            // Get the sizes of the items.
            SizeF base_size, power_size;
            float width, height;
            GetSizes(gr, font, out base_size, out power_size, out width, out height);

            // Calculate our size.
            return new SizeF(width, height);
        }

        // Draw the equation.
        public override void Draw(Graphics gr, Font font, Pen pen, Brush brush, float x, float y)
        {
            // Get the sizes of the items.
            SizeF base_size, power_size;
            float width, height;
            GetSizes(gr, font, out base_size, out power_size, out width, out height);

            // Draw the base.
            float base_y = y + power_size.Height / 2;
            Base.Draw(gr, font, pen, brush, x, base_y);

            // Draw the power.
            float power_x = x + base_size.Width;
            Power.Draw(gr, font, pen, brush, power_x, y);
        }

        // Return various sizes.
        private void GetSizes(Graphics gr, Font font, out SizeF base_size, out SizeF power_size, out float width, out float height)
        {
            base_size = Base.GetSize(gr, font);
            power_size = Power.GetSize(gr, font);
            width = base_size.Width + power_size.Width;
            height = base_size.Height + power_size.Height / 2;
        }
    }
}
