﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.Drawing;

namespace howto_draw_equations
{
    class RootEquation : Equation
    {
        // Gap between items and horizontal lines.
        private float ExtraHeight = 2;

        // Extra width of line under the index.
        private float ExtraWidth = 4;

        // The items to draw.
        private Equation Index, Radicand;

        // The angle for the radical sign.
        private float Angle = (float)(80 * Math.PI / 180);

        // Initialize the equation.
        public RootEquation(Equation index, Equation radicand)
        {
            Index = index;
            Radicand = radicand;
        }

        // Set font sizes for sub-equations.
        public override void SetFontSizes(float font_size)
        {
            FontSize = font_size;
            Index.SetFontSizes(font_size * 0.75f);
            Radicand.SetFontSizes(font_size);
        }

        // Return the equation's size.
        public override SizeF GetSize(Graphics gr, Font font)
        {
            // Get the sizes.
            SizeF index_size, radicand_size, our_size;
            GetSizes(gr, font, out index_size, out radicand_size, out our_size);

            return our_size;
        }

        // Calculate sizes.
        private void GetSizes(Graphics gr, Font font, out SizeF index_size, out SizeF radicand_size, out SizeF our_size)
        {
            // Get the sizes of the index and radicand.
            index_size = Index.GetSize(gr, font);
            radicand_size = Radicand.GetSize(gr, font);

            // See how tall we must be.
            float height = ExtraHeight + Math.Max(
                2 * index_size.Height,
                radicand_size.Height);

            // Calculate our width.
            float width = index_size.Width + radicand_size.Width +
                (float)Math.Cos(Angle) * 1.5f * height + ExtraWidth;

            // Set our size.
            our_size = new SizeF(width, height);
        }

        // Draw the equation.
        public override void Draw(Graphics gr, Font font, Pen pen, Brush brush, float x, float y)
        {
            // Get the sizes.
            SizeF index_size, radicand_size, our_size;
            GetSizes(gr, font, out index_size, out radicand_size, out our_size);

            // Draw the radical symbol.
            float x1 = x + index_size.Width + ExtraWidth;
            float x2 = x1 + (float)Math.Cos(Angle) * our_size.Height / 2;
            float x3 = x2 + (float)Math.Cos(Angle) * our_size.Height;
            float x4 = x + our_size.Width;
            float y1 = y + our_size.Height / 2 + ExtraHeight;
            float y2 = y + our_size.Height + ExtraHeight;
            float y3 = y;
            float y4 = y;
            PointF[] pts =
            {
                new PointF(x, y1), 
                new PointF(x1, y1), 
                new PointF(x2, y2),
                new PointF(x3, y3),
                new PointF(x4, y4),
            };
            gr.DrawLines(pen, pts);

            // Draw the index.
            float index_x = x + ExtraWidth;
            float index_y = y + (our_size.Height / 2 - index_size.Height) / 2;
            Index.Draw(gr, font, pen, brush, index_x, index_y);

            // Draw the radicand.
            float randicand_x = x3;
            float randicand_y = y + ExtraHeight + (our_size.Height - ExtraHeight - radicand_size.Height) / 2;
            Radicand.Draw(gr, font, pen, brush, randicand_x, randicand_y);
        }
    }
}
